#include "actmap.hh"

void ActMap::InitMap()
{
    map.clear();
    map.resize(As() * Cs() * Ms());
}

unsigned ActMap::MdInd(unsigned a, unsigned c, unsigned m) const
{
    return m + Ms() * (c + Cs() * a);
}

void ActMap::Set(unsigned a, unsigned c, unsigned m)
{
    map[MdInd(a, c, m)] = true;
}

void ActMap::SetAgentChan(const Agent &ag, const Channel &ch)
{
    unsigned ai = AgentIndex(ag);
    unsigned ci = ChannelIndex(ch);
    for(unsigned a=0; a<Ms(); ++a)
        Set(ai, ci, a);
}

void ActMap::SetChanMode(const Channel &ch, const Mode &md)
{
    unsigned mi = ModeIndex(md);
    unsigned ci = ChannelIndex(ch);
    for(unsigned a=0; a<As(); ++a)
        Set(a, ci, mi);
}

#if ORIGINALALGORITHM
void ActMap::BuildCrossMap()
{
    #define Scan(sumxy, x,xm, y,ym, z,zm) \
        sumxy.SetDims(xm, ym); \
        for(unsigned x=0; x<(xm); ++x) \
            for(unsigned y=0; y<(ym); ++x) \
            { \
                unsigned sum=0; \
                for(unsigned z=0; z<zm; ++z) \
                    if(map[MdInd(a,c,m)])++sum; \
                sumxy.at(x, y) = sum; \
            }
    Scan(sumac, a, As(), c, Cs(), m, Ms());
    Scan(sumam, a, As(), m, Ms(), c, Cs());
    Scan(sumcm, c, Cs(), m, Ms(), a, As());
    #undef Scan
    
    handled.clear();
    handled.resize(map.size());
}

unsigned ActMap::CrossValue(unsigned a, unsigned c, unsigned m) const
{
    return sumac.at(a, c) * sumam.at(a, m) * sumcm.at(c, m);
}
#endif

/*
in:
    0123456789ABCDEFGHIJKLMNO  crossvaluemap
   P.!.....!....!.....!...... 20 24       20 12
   Q.!..........X............ 10          10
   R.!.....!....!.....!...... 20 24       20 12
   S.......!..!!!.....!......    30 10 10 25 15
   T.!.....!..!!............. 20 24  8  8
   U.......X....X............    12       10
   V.!.....!................. 10 12 
out:
      7CI*PRS + 17CI*PR + C*PQRSU + 7AB*ST + 17*PRTV + 7C*PRSU
     
  tai 7CI*PRS + 17*PRTV + CI*PR + C*Q + AB*ST + 7C*U
  tai 7CI*PRS + 17CI*PR + 17*TV + C*Q + AB*ST + 7C*U
     

    0123456789ABCDEFGHIJKLMNO
   P.X.....X....X.....X......    3 *(6+4+3) = 39 13      4,0 6,3 -,- -,- 4,3 3,3
   Q.X..........X............                            0,0 -,- -,- -,- 0,- -,-
   R.X.....X....X.....X......    3 *(6+4+3) = 39 13      4,0 6,3 -,- -,- 4,3 3,3
   S.......X..XXX.....X......    5 *(6+4+3) = 39 13      -,- 6,5 2,5 2,5 4,5 3,5
   T.X.....X..XX.............    4 *(6+2+2) = 30 10      4,0 6,4 2,4 2,4 -,- -,-
   U.......X....X............    2 *(6+4) = 20   10      -,- 6,2 -,- -,- 4,2 -,-
   V.X.....X.................    2 *(4+6) = 20   10      4,0 6,2 -,- -,- -,- -,-
   
     4     6  224     3
      (3+3+3+1) =  40  (3+3+3) = 27
                 (3+3+3+2) = 44
                (3+3) = 12
               (3+3) =12
            (3+3+3+3+2+1) = 90
      10 15 6 6 11 9
  
    4  6  2  2  4  3        10  15  6  6  11   9        40   90  12  12   44   27
   12`18  6  6`12 `9 3     130 195       143 117 13   1560 3510 468 468 1716 1053 39
   12`18  6  6`12 `9 3     130 195       143 117 13   1560 3510 468 468 1716 1053 39
   20`30 10 10`20`15 5         195 78 78     117 13   1560 3510 468 468 1716 1053 39
   12 18  6  6 12  9 3     100 150 60 60 110     10   1200 2700 360 360 1320  810 30
    8 12  4  4  8  6 2         150       110     10    800 1800 240 240  880  540 20
    4  6  2  2  4  3 1     100 150               10    800 1800 240 240  880  540 20

 Testi... Etsitn kaikki suorat pisteet jotka
 ovat samalla suoralla 7P:n kanssa, ja niden pisteiden
 kanssa samalla suoralla olevat pisteet, jotka ovat samalla suoralla
 jonkin sellaisen pisteen kanssa, joka on samalla suoralla 7P:n kanssa:
      1*P + 7*P + C*P + I*P
      1*Q + C*Q
      1*R + 7*R + C*R + I*R
      1*S + 7*S + C*S + I*S
            7*U
 
     =    17CP
         Pxxxx 
         Qx.x.
         Rxxxx
         Sxxxx
         U.xx.
     Etsitn isoin ryhm:
     Vaihtoehtoisia ryhmi:
        1C*PQRS  (=8)
        C*PQRSU  (=5)
        17CP*PRS (=12)
        7C*RSU   (=6)


         ,-----#kaupunki
         |,----#gurb
         ||,---#pkmn
         |||,--#kerava
  Bisqwit....
      Dih.X.X     Dih tai Ana: #gurb,#kerava
  BisqBot...X     BisqBot: #kerava
     Ana_.X.X
*/

#if ORIGINALALGORITHM
#include <set>
#endif

static bool SamallaSuoralla(unsigned a,unsigned c,unsigned m,
                            unsigned A,unsigned C,unsigned M)
{
	return ((a==A && c==C) || (a==A && m==M) || (c==C && m==M));
}

void ActMap::Squarify()
{
#if ORIGINALALGORITHM
    BuildCrossMap();
    
    typedef multiset<point3d> valueset;
    
    valueset values;
    for(unsigned a=0; a<As(); ++a)
        for(unsigned c=0; c<Cs(); ++c)
            for(unsigned m=0; m<Ms(); ++m)
                values.insert(point3d(CrossValue(a,c,m), a,c,m));

    valueset::const_iterator i;
    for(i=values.begin(); i!=values.end() ++i)
    {
        point3d &p = *i;
        unsigned a = p.x;
        unsigned c = p.y;
        unsigned m = p.z;
        if(handled[MdInd(a, c, m)])continue;
        
        unsigned aa=0;
        for(unsigned A=0; A<As(); ++A) if(map[MdInd(A, c, m)])
        {
        	++aa;
        	unsigned cc=0;
        	for(unsigned C=0; C<As(); ++C) if(map[MdInd(A, C, m)] && map[MdInd(a, C, m)])
        	{
        		++cc;
        	}
        }
    }
#else
	vector<elem> elems;

    unsigned setcount = 0;
    for(unsigned a=0; a<As(); ++a)
        for(unsigned c=0; c<Cs(); ++c)
            for(unsigned m=0; m<Ms(); ++m)
                if(map[MdInd(a,c,m)])
                {
                    ++setcount;
                    elems.push_back(elem(a, c, m));
                }
	for(;;)
	{
	}
#endif
}
