#include <vector>
#include <iostream>
#include <cstdlib>

using std::vector;
using std::exit;
using std::cout;

#include "actmap.hh"

static ActMap map;

void AgentChan(const string &ag, const string &ch, bool first)
{
    if(first)
    {
        map.AddAgent(ag);
        map.AddChannel(ch);
    }
    else
        map.SetAgentChan(ag, ch);
}

/* parses "dest[,dest[...]] mode [parm[ parm[...]]]" */
void DoMode(const string &str, bool first)
{
    vector<Channel> chlist;
    
    unsigned a, b=str.size(), c;
    for(a=0; a<b; ++a)
    {
        c=a;
        while(a<b && str[a]!=' ' && str[a]!=',')++a;
        
        string s = str.substr(c, a-c);
        if(!first)chlist.push_back(s);
        map.AddChannel(s);
        
        if(a >= b || str[a]==' ')break;
    }
    while(a<b && str[a]==' ')++a;
    for(c=a; a<b && str[a]!=' '; ++a);

    vector<string> parms;
    string mstr = str.substr(c, a-c);
    for(; a<b; ++a)
    {
        for(c=a; a<b && str[a]!=' ' && str[a]!=','; ++a);
        parms.push_back(str.substr(c, a-c));
        if(a >= b || str[a]==' ')break;
    }
    bool nega = false;
    unsigned p=0;
    for(a=0, b=mstr.size(); a<b; ++a)
    {
        if(mstr[a]=='+') { nega=false; continue; }
        if(mstr[a]=='-') { nega=true; continue; }
        Mode tmp(nega, mstr[a]);
        if(tmp.needsparm() && p < parms.size())
        {
            tmp.setparm(parms[p++]);
        }
        if(first)
            map.AddMode(tmp);
        else
        {
            for(unsigned d=0; d<chlist.size(); ++d)
                map.SetChanMode(chlist[d], tmp);
        }
    }
}

static void Usage()
{
	cout <<
	    "IRC Mode combiner v"VERSION" for agent system like in BisqBot\n"
	    "Copyright (C) 1992,2001 Bisqwit (http://iki.fi/bisqwit/)\n"
	    "\n"
	    "modecombiner <agentdef> [<...>] / [<modedef>]\n"
	    "\n"
	    "\tagentdef is: <nick> <channellist>\n"
	    "\tmodedef is: <modestring>\n"
	    "\tchannellist is: \"<chan> [<...>]\"\n"
	;
	exit(0);
}
/* Example: modecombiner 
      Bisqwit "#chan #pruu"
      Dih "#blaa #bluu"
      /
      "mode1"
      "mode2"
*/
static void ParseArgs(int argc, const char *const *argv, bool isfirst)
{
    int v;
    for(v=1; v<argc; ++v)
    {
    	if(argv[v][0] == '/' && !argv[v][0]) { ++v; break; }
    	
        string agnam = argv[v];
        string s = argv[++v];

        for(unsigned a=0, b=s.size(); a<b; ++a)
        {
            unsigned c = a;
            while(a<b && s[a]!=' ' && s[a]!=',')++a;
            
            if(a > c)
            {
	            string s = s.substr(c, a-c);
	            AgentChan(agnam, s, isfirst);
	        }
        }
    }
    if(v == argc)
    {
    	Usage();
    }
    while(v < argc)DoMode(argv[v++], isfirst);
}

int main(int argc, const char *const *argv)
{
	/* List all channels and agents and modes. */
	ParseArgs(argc, argv, true);
    map.InitMap();
    
	/* Now build the map. */
	ParseArgs(argc, argv, false);
	
	//Optimize();

	return 0;
}
